// Support_GridlessBricks
// Adds support for overlap detection for gridless bricks
//
// To use, set %datablock.isGridless = 1; in the brick datablock
// For bricks that have multiple states that need overlap detection for the base brick (like doors):
//		set %datablock.gridlessID = "identifier string";
//		eg. prevents "place button brick, click once to set to pressed datablock, place a new unpressed button brick, repeat" to stack infinite bricks on the same spot.
// For bricks that should not have bounding boxes overlap with conflicting bricks:
//		set %datablock.boxSearch = 1;
//		eg. prevents a 4x4 gridless cube from overlapping other 4x4 gridless cubes, while allowing it to overlap any other gridded/gridless brick.
// For bricks that should be allowed to overlap when rotated:
//		set %datablock.doRotationalCheck = 1;
//		eg. allows a gridless corner brick to be rotated and placed on top of itself to cover all 4 corners.

//Changelog
// V1.1 -> V1.2
// - Delay activating package to ensure this comes on top of everything
// V1 -> V1.1
// - Add setting to determine if a box search or radius search is used
// - Add changelog, documentation on how to use

if ($Support_GridlessBricks::Version > 1.1)
{
	return;
}

$Support_GridlessBricks::Version = 1.1;


if (isFunction(RTB_registerPref))
{
	RTB_registerPref("Allow non-admins to place gridless bricks",		"Gridless Bricks",	"$Pref::Server::GridlessBricks::AllowNonAdmin",	"bool", "Support_GridlessBricks",	0,	0,	0);	
}
else
{
	if ($Pref::Server::GridlessBricks::AllowNonAdmin $= "") $Pref::Server::GridlessBricks::AllowNonAdmin = 0;
}

package GridlessBricks
{
	//add events if we're a door brick, only in onPlant
	function fxDTSBrick::plant(%obj)
	{
		%data = %obj.getDataBlock();

		//apply events
		if (%data.isGridless)
		{
			if (isObject(%obj.client) && !%obj.client.isAdmin && !$Pref::Server::GridlessBricks::AllowNonAdmin)
			{
				%obj.client.centerprint("Only admins are allowed to place gridless bricks", 1);
				return 6;
			}
			if (%data.boxSearch)
			{
				initContainerBoxSearch(%obj.position, _getContainerBoxSearchSize(%obj), $Typemasks::fxBrickAlwaysObjectType);
			}
			else
			{
				initContainerRadiusSearch(%obj.position, 0.01, $Typemasks::fxBrickAlwaysObjectType);
			}

			while (isObject(%next = containerSearchNext()))
			{
				%nextDB = %next.getDatablock();
				if (%next == %obj || %next.isDead())
				{
					continue;
				}

				// if (vectorDist(%next.position, %obj.position) < 0.1
				// 	&& (%nextDB == %data || (%nextDB.gridlessID $= %data.gridlessID && %nextDB.gridlessID !$= "")))
				if (%nextDB == %data || (%nextDB.gridlessID $= %data.gridlessID && %nextDB.gridlessID !$= ""))
				{
					if (%data.doRotationalCheck && %next.getAngleID() == %obj.getAngleID())
					{
						return 1;
					}
					else if (!%data.doRotationalCheck)
					{
						return 1;
					}
				}
			}
		}
		return parent::plant(%obj);
	}
};
schedule(1000, 0, activatePackage, GridlessBricks);

function _getContainerBoxSearchSize(%brick)
{
	%box = %brick.getWorldBox();
	%small = getWords(%box, 0, 2);
	%large = getWords(%box, 3, 5);
	%dimX = getWord(%large, 0) - getWord(%small, 0);
	%dimY = getWord(%large, 1) - getWord(%small, 1);
	%dimZ = getWord(%large, 2) - getWord(%small, 2);
	return %dimX - 0.01 SPC %dimY - 0.01 SPC %dimZ - 0.01;
}